from mosparo_django.config import FrontendConfig

def test_frontend_config():
    fc = FrontendConfig('https://test.local', 'test_uuid', 'test_public_key')

    assert 'https://test.local' == fc.get_host()
    assert 'test_uuid' == fc.get_uuid()
    assert 'test_public_key' == fc.get_public_key()


from django.forms import Form, CharField

from mosparo_django.forms import MosparoForm
from mosparo_django.fields import MosparoField

def test_mosparo_form():
    class DummyMosparoField(MosparoField):
        was_called = False

        def verify_data(self, form: Form):
            self.was_called = True

    class CustomForm(MosparoForm):
        name = CharField(label='Name', max_length=255)
        mosparo = DummyMosparoField()

    form = CustomForm({
        'name': 'Test'
    })

    form.is_valid()

    assert form.fields['mosparo'].was_called is True

def test_mosparo_form_without_field():
    class CustomForm(MosparoForm):
        name = CharField(label='Name', max_length=255)

    form = CustomForm({
        'name': 'Test'
    })

    res = form.is_valid()

    # Form should be invalid since no mosparo field was added to the form
    assert res is False



from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError

from mosparo_api_client import Client
from mosparo_django.config import FrontendConfig
from mosparo_django.widgets import MosparoWidget


class MosparoField(forms.BooleanField):
    widget = MosparoWidget

    mosparo_host = None
    mosparo_uuid = None
    mosparo_public_key = None
    mosparo_private_key = None
    mosparo_verify_ssl = True

    callback_ignored_field_types = None
    callback_verifiable_field_types = None
    callback_after_prepare_form_data = None

    def __init__(self, mosparo_host=None, mosparo_uuid=None, mosparo_public_key=None, mosparo_private_key=None,
                 mosparo_verify_ssl=None, callback_ignored_field_types=None, callback_verifiable_field_types=None,
                 callback_after_prepare_form_data=None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if not isinstance(self.widget, MosparoWidget):
            raise ImproperlyConfigured(
                "The widget must be a subclass of mosparo_django.widgets.MosparoWidget"
            )

        if mosparo_host is None:
            mosparo_host = getattr(settings, 'MOSPARO_HOST', '')

        if mosparo_uuid is None or mosparo_public_key is None or mosparo_private_key is None:
            mosparo_uuid = getattr(settings, 'MOSPARO_UUID', '')
            mosparo_public_key = getattr(settings, 'MOSPARO_PUBLIC_KEY', '')
            mosparo_private_key = getattr(settings, 'MOSPARO_PRIVATE_KEY', '')

        if mosparo_verify_ssl is None:
            mosparo_verify_ssl = getattr(settings, 'MOSPARO_VERIFY_SSL', True)

        self.mosparo_host = mosparo_host
        self.mosparo_uuid = mosparo_uuid
        self.mosparo_public_key = mosparo_public_key
        self.mosparo_private_key = mosparo_private_key
        self.mosparo_verify_ssl = mosparo_verify_ssl

        self.callback_ignored_field_types = callback_ignored_field_types
        self.callback_verifiable_field_types = callback_verifiable_field_types
        self.callback_after_prepare_form_data = callback_after_prepare_form_data

        frontend_config = FrontendConfig(self.mosparo_host, self.mosparo_uuid, self.mosparo_public_key)
        self.widget.set_frontend_config(frontend_config)

        # Hide the label for the field, important for the invisible mode.
        self.label = ''

    def validate(self, value):
        return True

    def verify_data(self, form: forms.Form):
        api_client = Client(
            self.mosparo_host,
            self.mosparo_public_key,
            self.mosparo_private_key,
            self.mosparo_verify_ssl
        )

        if '_mosparo_submitToken' not in form.data or '_mosparo_validationToken' not in form.data:
            raise ValidationError('mosparo specific fields were not found in the submitted data.')

        data = self.prepare_form_data(form)
        submit_token = form.data['_mosparo_submitToken']
        validation_token = form.data['_mosparo_validationToken']

        res = api_client.verify_submission(data['form_data'], submit_token, validation_token)

        if res != None:
            verified_fields = res.get_verified_fields()
            required_field_difference = set(data['required_fields']) - set(verified_fields.keys())
            verifiable_field_difference = set(data['verifiable_fields']) - set(verified_fields.keys())

            # The submission is only valid if all required and verifiable fields got verified
            if res.is_submittable() and not required_field_difference and not verifiable_field_difference:
                form.valid = True
                return

        form.valid = False
        raise ValidationError('mosparo could not verify the data. Please check the submission in mosparo.')

    def get_mosparo_host(self):
        return self.mosparo_host

    def get_mosparo_uuid(self):
        return self.mosparo_uuid

    def get_mosparo_public_key(self):
        return self.mosparo_public_key

    def prepare_form_data(self, form: forms.Form):
        data = { 'form_data': {}, 'required_fields': [], 'verifiable_fields': [] }
        ignored_field_types = [
            'PasswordInput',
            'HiddenInput',
            'MultipleHiddenInput',
            'FileInput',
            'ClearableFileInput',
            'MosparoField',
        ]
        if self.callback_ignored_field_types != None:
            ignored_field_types = self.callback_ignored_field_types(ignored_field_types)
        
        verifiable_field_types = [
            'TextInput',
            'Textarea',
            'EmailInput',
            'URLInput',
        ]
        if self.callback_verifiable_field_types != None:
            verifiable_field_types = self.callback_verifiable_field_types(verifiable_field_types)

        for key, field in form.fields.items():
            if type(field).__name__ in ignored_field_types or type(field.widget).__name__ in ignored_field_types:
                continue

            name = key

            if field.required:
                data['required_fields'].append(name)

            if type(field).__name__ in verifiable_field_types or type(field.widget).__name__ in verifiable_field_types:
                data['verifiable_fields'].append(name)

            if key in form.data:
                data['form_data'][key] = form.data[key]

        if self.callback_after_prepare_form_data != None:
            data = self.callback_after_prepare_form_data(data)

        return data


class FrontendConfig:
    host: str = None
    uuid: str = None
    public_key: str = None

    def __init__(self, host: str, uuid: str, public_key: str):
        self.host = host
        self.uuid = uuid
        self.public_key = public_key

    def get_host(self):
        return self.host

    def get_uuid(self):
        return self.uuid

    def get_public_key(self):
        return self.public_key


import uuid

from django.forms import widgets

from mosparo_django.config import FrontendConfig

class MosparoWidget(widgets.Widget):
    template_name = 'mosparo/box.html'

    frontend_config = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.uuid = uuid.uuid4().hex

    def set_frontend_config(self, frontend_config: FrontendConfig):
        self.frontend_config = frontend_config

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context.update(
            {
                'widget_uuid': self.uuid,
                'mosparo_host': self.frontend_config.get_host(),
                'mosparo_uuid': self.frontend_config.get_uuid(),
                'mosparo_public_key': self.frontend_config.get_public_key()
            }
        )
        return context



from django.forms import Form, Widget, CharField, URLField, PasswordInput, TextInput
from django.core.exceptions import ImproperlyConfigured, ValidationError
import pytest
from django.http import QueryDict

from mosparo_api_client import RequestHelper, VerificationResult

from mosparo_django.fields import MosparoField

def test_mosparo_field():
    mf = MosparoField()

    assert 'https://host.local' == mf.get_mosparo_host()
    assert 'uuid' == mf.get_mosparo_uuid()
    assert 'public_key' == mf.get_mosparo_public_key()
    assert mf.validate(1) is True

def test_mosparo_field_custom_args():
    mf = MosparoField(mosparo_host='https://custom-test.local', mosparo_uuid='custom_uuid',
                      mosparo_public_key='custom_public_key', mosparo_private_key='custom_private_key')

    assert 'https://custom-test.local' == mf.get_mosparo_host()
    assert 'custom_uuid' == mf.get_mosparo_uuid()
    assert 'custom_public_key' == mf.get_mosparo_public_key()
    assert mf.validate(1) is True

    assert 'https://custom-test.local' == mf.widget.frontend_config.get_host()
    assert 'custom_uuid' == mf.widget.frontend_config.get_uuid()
    assert 'custom_public_key' == mf.widget.frontend_config.get_public_key()

def test_mosparo_field_custom_widget():
    class CustomMosparoField(MosparoField):
        widget = Widget

    with pytest.raises(ImproperlyConfigured) as exc:
        mf = CustomMosparoField()

    assert 'The widget must be a subclass of mosparo_django.widgets.MosparoWidget' in str(exc.value)

def test_mosparo_field_verify_data_without_mosparo_tokens():
    class DummyForm(Form):
        data = {'name': 'Test'}

    form = DummyForm()
    mf = MosparoField()

    with pytest.raises(ValidationError) as exc:
        mf.verify_data(form)

    assert 'mosparo specific fields were not found in the submitted data.' in str(exc.value)

def test_mosparo_field_verify_data(requests_mock):
    # Prepare the form
    class DummyForm(Form):
        name = CharField(label='Name', max_length=255)
        password = CharField(label='Password', max_length=255, widget=PasswordInput)

    data = QueryDict('name=Test&_mosparo_submitToken=submitToken&_mosparo_validationToken=validationToken')
    form = DummyForm(data=data)

    form.is_valid()

    # Prepare the API request
    public_key = 'public_key'
    private_key = 'private_key'
    validation_token = 'validationToken'
    form_data = {'name': 'Test'}

    request_helper = RequestHelper(public_key, private_key)

    prepared_form_data = request_helper.prepare_form_data(form_data)
    form_signature = request_helper.create_form_data_hmac_hash(prepared_form_data)

    validation_signature = request_helper.create_hmac_hash(validation_token)
    verification_signature = request_helper.create_hmac_hash(validation_signature + form_signature)

    requests_mock.post('https://host.local/api/v1/verification/verify', json={
        'valid': True,
        'verificationSignature': verification_signature,
        'verifiedFields': {'name': VerificationResult.FIELD_VALID},
        'issues': []
    }, status_code=200)

    mf = MosparoField()
    mf.verify_data(form)

def test_mosparo_field_verify_data_with_ignored_callback(requests_mock):
    # Prepare the form
    class DummyForm(Form):
        name = CharField(label='Name', max_length=255)
        password = CharField(label='Password', max_length=255, widget=PasswordInput)

    data = QueryDict('name=Test&_mosparo_submitToken=submitToken&_mosparo_validationToken=validationToken')
    form = DummyForm(data=data)

    form.is_valid()

    # Prepare the API request
    public_key = 'public_key'
    private_key = 'private_key'
    validation_token = 'validationToken'
    form_data = {'name': 'Test'}

    request_helper = RequestHelper(public_key, private_key)

    prepared_form_data = request_helper.prepare_form_data(form_data)
    form_signature = request_helper.create_form_data_hmac_hash(prepared_form_data)

    validation_signature = request_helper.create_hmac_hash(validation_token)
    verification_signature = request_helper.create_hmac_hash(validation_signature + form_signature)

    requests_mock.post('https://host.local/api/v1/verification/verify', json={
        'valid': True,
        'verificationSignature': verification_signature,
        'verifiedFields': {'name': VerificationResult.FIELD_VALID},
        'issues': []
    }, status_code=200)

    def test_callback(types):
        return []

    mf = MosparoField(callback_ignored_field_types=test_callback)

    with pytest.raises(ValidationError) as exc:
        # Fails because the password field was not ignored and was missing in the list of verified fields
        mf.verify_data(form)

    assert 'mosparo could not verify the data. Please check the submission in mosparo.' in str(exc.value)

def test_mosparo_field_verify_data_with_verifiable_callback(requests_mock):
    class TestInput(TextInput):
        pass

    # Prepare the form
    class DummyForm(Form):
        name = CharField(label='Name', max_length=255)
        url = URLField(label='URL', max_length=255, required=False)
        test_field = CharField(label='Test', max_length=255, required=False, widget=TestInput)

    data = QueryDict('name=Test&_mosparo_submitToken=submitToken&_mosparo_validationToken=validationToken')
    form = DummyForm(data=data)

    form.is_valid()

    # Prepare the API request
    public_key = 'public_key'
    private_key = 'private_key'
    validation_token = 'validationToken'
    form_data = {'name': 'Test'}

    request_helper = RequestHelper(public_key, private_key)

    prepared_form_data = request_helper.prepare_form_data(form_data)
    form_signature = request_helper.create_form_data_hmac_hash(prepared_form_data)

    validation_signature = request_helper.create_hmac_hash(validation_token)
    verification_signature = request_helper.create_hmac_hash(validation_signature + form_signature)

    requests_mock.post('https://host.local/api/v1/verification/verify', json={
        'valid': True,
        'verificationSignature': verification_signature,
        'verifiedFields': {'name': VerificationResult.FIELD_VALID, 'url': VerificationResult.FIELD_VALID},
        'issues': []
    }, status_code=200)

    def test_callback(types):
        # Add TestInput to the list of verifiable fields
        return types + ['TestInput']

    mf = MosparoField(callback_verifiable_field_types=test_callback)

    with pytest.raises(ValidationError) as exc:
        mf.verify_data(form)

    # Fails, because not all verifiable fields got verified (the TestInput field type was missing)
    assert 'mosparo could not verify the data. Please check the submission in mosparo.' in str(exc.value)

def test_mosparo_field_verify_data_with_after_data_callback(requests_mock):
    class TestInput(TextInput):
        pass

    # Prepare the form
    class DummyForm(Form):
        name = CharField(label='Name', max_length=255)
        url = URLField(label='URL', max_length=255, required=False)
        test_field = CharField(label='Test', max_length=255, required=False, widget=TestInput)

    data = QueryDict('name=Test&_mosparo_submitToken=submitToken&_mosparo_validationToken=validationToken')
    form = DummyForm(data=data)

    form.is_valid()

    # Prepare the API request
    public_key = 'public_key'
    private_key = 'private_key'
    validation_token = 'validationToken'
    form_data = {'name': 'Test'}

    request_helper = RequestHelper(public_key, private_key)

    prepared_form_data = request_helper.prepare_form_data(form_data)
    form_signature = request_helper.create_form_data_hmac_hash(prepared_form_data)

    validation_signature = request_helper.create_hmac_hash(validation_token)
    verification_signature = request_helper.create_hmac_hash(validation_signature + form_signature)

    requests_mock.post('https://host.local/api/v1/verification/verify', json={
        'valid': True,
        'verificationSignature': verification_signature,
        'verifiedFields': {'name': VerificationResult.FIELD_VALID, 'url': VerificationResult.FIELD_VALID},
        'issues': []
    }, status_code=200)

    def test_callback(data):
        data['required_fields'].append('test-field')
        return data

    mf = MosparoField(callback_after_prepare_form_data=test_callback)

    with pytest.raises(ValidationError) as exc:
        mf.verify_data(form)

    # Fails, because
    assert 'mosparo could not verify the data. Please check the submission in mosparo.' in str(exc.value)

def test_mosparo_field_verify_data_validation_error(requests_mock):
    # Prepare the form
    class DummyForm(Form):
        name = CharField(label='Name', max_length=255)

    data = QueryDict('name=Test&_mosparo_submitToken=submitToken&_mosparo_validationToken=validationToken')
    form = DummyForm(data=data)

    form.is_valid()

    # Prepare the API request
    public_key = 'public_key'
    private_key = 'private_key'
    validation_token = 'validationToken'
    form_data = {'name': 'Test'}

    request_helper = RequestHelper(public_key, private_key)

    prepared_form_data = request_helper.prepare_form_data(form_data)
    form_signature = request_helper.create_form_data_hmac_hash(prepared_form_data)

    validation_signature = request_helper.create_hmac_hash(validation_token)
    verification_signature = request_helper.create_hmac_hash(validation_signature + form_signature)

    requests_mock.post('https://host.local/api/v1/verification/verify', json={
        'valid': False,
        'verificationSignature': verification_signature,
        'verifiedFields': {'name': VerificationResult.FIELD_VALID},
        'issues': []
    }, status_code=200)

    mf = MosparoField()

    with pytest.raises(ValidationError) as exc:
        mf.verify_data(form)

    assert 'mosparo could not verify the data. Please check the submission in mosparo.' in str(exc.value)




from django import forms
from django.core.exceptions import ValidationError

from .fields import MosparoField

class MosparoForm(forms.Form):
    def clean(self):
        mosparo_field = None
        for key, field in self.fields.items():
            if isinstance(field, MosparoField):
                mosparo_field = field
                break

        if mosparo_field is None:
            raise ValidationError('No mosparo field found in form.')

        mosparo_field.verify_data(self)

        return super().clean()



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

__project__ = 'mosparo_django'
__version__ = "1.0.0b2"

settings_types = {
    'MOSPARO_HOST': str,
    'MOSPARO_UUID': str,
    'MOSPARO_PUBLIC_KEY': str,
    'MOSPARO_PRIVATE_KEY': str,
    'MOSPARO_VERIFY_SSL': bool
}

required_settings = ['MOSPARO_HOST', 'MOSPARO_UUID', 'MOSPARO_PUBLIC_KEY', 'MOSPARO_PRIVATE_KEY']

for setting, type in settings_types.items():
    if not hasattr(settings, setting) and setting in required_settings:
        raise ImproperlyConfigured('Setting "%s" is not defined.' % setting)

    if hasattr(settings, setting) and not isinstance(getattr(settings, setting), type):
        raise ImproperlyConfigured('Type of Setting "%s" is not correct. Should be "%s".' % setting, type)


from mosparo_django.config import FrontendConfig
from mosparo_django.widgets import MosparoWidget

def test_mosparo_widget():
    fc = FrontendConfig('https://test.local', 'test_uuid', 'test_public_key')

    mw = MosparoWidget()
    mw.set_frontend_config(fc)

    context = mw.get_context('test', {}, {})

    assert 'https://test.local' == context['mosparo_host']
    assert 'test_uuid' == context['mosparo_uuid']
    assert 'test_public_key' == context['mosparo_public_key']
    assert 'mosparo/box.html' == mw.template_name
